@@ -1,12 +1,18 @@
+import base64
 import collections
 import contextlib
-import json
+import simplejson as json
 import os
+import re
 import requests
 import shutil
 import tarfile
 import tempfile
 
+__version__ = '0.2.0'
+
+requests.packages.urllib3.disable_warnings()
+
 
 @contextlib.contextmanager
 def pushd(path):
@@ -22,7 +28,8 @@ class GistInfo(collections.namedtuple('GistInfo', 'id public desc')):
 
 class authenticate(object):
     """
-    The class is used as a decorator to handle token authentication with github.
+    The class is used as a decorator to handle token authentication with
+    github.
     """
 
     def __init__(self, func, method='GET'):
@@ -39,7 +46,7 @@ class authenticate(object):
         self.headers = {
                 'Accept-Encoding': 'identity, deflate, compress, gzip',
                 'User-Agent': 'python-requests/1.2.0',
-                'Accept': '*/*',
+                'Accept': 'application/vnd.github.v3.base64',
                 }
         self.method = method
 
@@ -63,6 +70,16 @@ class authenticate(object):
         """
         return cls(func, method='POST')
 
+    @classmethod
+    def patch(cls, func):
+        """Create an authenticate object with a PATCH method
+
+        Arguments:
+            func: a function to decorate
+
+        """
+        return cls(func, method='PATCH')
+
     @classmethod
     def delete(cls, func):
         """Create an authenticate object with a DELETE method
@@ -94,7 +111,12 @@ class authenticate(object):
         try:
             url = 'https://api.github.com/gists'
             params = {'access_token': self.instance.token}
-            request = requests.Request(self.method, url, headers=self.headers, params=params)
+            request = requests.Request(
+                    self.method,
+                    url,
+                    headers=self.headers,
+                    params=params,
+                    )
             return self.func(self.instance, request, *args, **kwargs)
         finally:
             self.instance = None
@@ -129,26 +151,73 @@ class GistAPI(object):
 
         """
         if stem is not None:
+            assert not stem.startswith('/')
             request.url = os.path.join(request.url, stem)
         return requests.Session().send(request.prepare())
 
-    @authenticate.get
-    def list(self, request):
+    def list(self):
         """Returns a list of the users gists as GistInfo objects
 
-        Arguments:
-            request: an initial request object
-
         Returns:
             a list of GistInfo objects
 
         """
-        gists = self.send(request).json()
-        info = []
-        for gist in gists:
-            info.append(GistInfo(gist['id'], gist['public'], gist['description']))
+        # Define the basic request. The per_page parameter is set to 100, which
+        # is the maximum github allows. If the user has more than one page of
+        # gists, this request object will be modified to retrieve each
+        # successive page of gists.
+        request = requests.Request(
+                'GET',
+                'https://api.github.com/gists',
+                headers={
+                    'Accept-Encoding': 'identity, deflate, compress, gzip',
+                    'User-Agent': 'python-requests/1.2.0',
+                    'Accept': 'application/vnd.github.v3.base64',
+                    },
+                params={
+                    'access_token': self.token,
+                    'per_page': 100,
+                    },
+                )
 
-        return info
+        # Github provides a 'link' header that contains information to
+        # navigate through a users page of gists. This regex is used to
+        # extract the URLs contained in this header, and to find the next page
+        # of gists.
+        pattern = re.compile(r'<([^>]*)>; rel="([^"]*)"')
+
+        gists = []
+        while True:
+            # Retrieve the next page of gists
+            response = self.send(request)
+            for gist in response.json():
+                gists.append(
+                        GistInfo(
+                            gist['id'],
+                            gist['public'],
+                            gist['description'],
+                            )
+                        )
+
+            try:
+                link = response.headers['link']
+
+                # Search for the next page of gist. If a 'next' page is found,
+                # the URL is set to this new page and the iteration continues.
+                # If there is no next page, return the list of gists.
+                for result in pattern.finditer(link):
+                    url = result.group(1)
+                    rel = result.group(2)
+                    if rel == 'next':
+                        request.url = url
+                        break
+                else:
+                    return gists
+
+            except Exception:
+                break
+
+        return gists
 
     @authenticate.post
     def create(self, request, desc, files, public=False):
@@ -224,15 +293,23 @@ class GistAPI(object):
 
         """
         gist = self.send(request, id).json()
-        return {name: data['content'] for name, data in gist['files'].items()}
+
+        def convert(data):
+            return base64.b64decode(data).decode('utf-8')
+
+        content = {}
+        for name, data in gist['files'].items():
+            content[name] = convert(data['content'])
+
+        return content
 
     @authenticate.get
     def archive(self, request, id):
         """Create an archive of a gist
 
         The files in the gist are downloaded and added to a compressed archive
-        (tarball). If the ID of the gist was c78d925546e964b4b1df, the resulting
-        archive would be,
+        (tarball). If the ID of the gist was c78d925546e964b4b1df, the
+        resulting archive would be,
 
             c78d925546e964b4b1df.tar.gz
 
@@ -247,7 +324,7 @@ class GistAPI(object):
 
         with tarfile.open('{}.tar.gz'.format(id), mode='w:gz') as archive:
             for name, data in gist['files'].items():
-                with tempfile.NamedTemporaryFile() as fp:
+                with tempfile.NamedTemporaryFile('w+') as fp:
                     fp.write(data['content'])
                     fp.flush()
                     archive.add(fp.name, arcname=name)
@@ -258,8 +335,8 @@ class GistAPI(object):
 
         The files in the gist a cloned to a temporary directory and passed to
         the default editor (defined by the EDITOR environmental variable). When
-        the user exits the editor, they will be provided with a prompt to commit
-        the changes, which will then be pushed to the remote.
+        the user exits the editor, they will be provided with a prompt to
+        commit the changes, which will then be pushed to the remote.
 
         Arguments:
             request: an initial request object
@@ -278,6 +355,34 @@ class GistAPI(object):
             finally:
                 shutil.rmtree(id)
 
+    @authenticate.post
+    def fork(self, request, id):
+        """Fork a gist
+
+        Forks an existing gist.
+
+        Arguments:
+            request: an initial request object
+            id:      the gist identifier
+
+        """
+        return self.send(request, '{}/forks'.format(id))
+
+    @authenticate.patch
+    def description(self, request, id, description):
+        """Updates the description of a gist
+
+        Arguments:
+            request:     an initial request object
+            id:          the id of the gist we want to edit the description for
+            description: the new description
+
+        """
+        request.data = json.dumps({
+            "description": description
+        })
+        return self.send(request, id).json()['html_url']
+
     def clone(self, id, name=None):
         """Clone a gist
 
